@@ -3,7 +3,7 @@ Models
 ------
 """
 
-from ..logging import get_logger
+from darts.logging import get_logger
 
 logger = get_logger(__name__)
 
